"""A module that defines constants of expression's variable names.
"""

from typing_extensions import Final

DISPLAY_OBJECT: Final[str] = 'display_object'
GRAPHICS: Final[str] = 'graphics'
RECTANGLE: Final[str] = 'rectangle'
SPRITE: Final[str] = 'sprite'
ARRAY: Final[str] = 'array'
INDEX: Final[str] = 'index'
BOOLEAN: Final[str] = 'boolean'
INT: Final[str] = 'int'
NUMBER: Final[str] = 'number'
STRING: Final[str] = 'string'
